/***************************************************************************/
/*                                                                         */
/* File:   button.cpp                                                      */
/* Author: bkenwright@xbdev.net                                            */
/* Date:   19-12-2005 (xmas)                                               */
/*                                                                         */
/***************************************************************************/

#include "button.h"


// Define an alpha value for our buttons, so that we can specify how
// transparent our buttons are on the screen. 1.0f is non-transparent, and
// 0.5 is 50% transparent.
#define					BUTTONALPHA						0.4f
#define					BUTTON_FONT_SIZE				15

// Give each button its own unique ID
int						UIButton::GlobalRef				= 0;

// Linked list of buttons
Button*					UIButton::pButtonList			= NULL;

// Mouse coordinates and settings
Mouse					UIButton::TheMouse				= {0,0,0,0,0};

// Auto-Cleanup Idea - so that any memory/resources allocated in our 
// linked list are release when our program ends.  As UIAutoButtonDestroy
// variable CleanUp destructor will be called when the program ends
// Remember its a static variable!
UIAutoButtonDestroy		UIButton::CleanUp;



// MouseActions(..)
// One of the most important functions, as it is called from our app
// to update and set all our Mouse informatoin, movement, button
// presses etc.
void UIButton::MouseActions(int x,   // y position of mouse
							int y,   // x position of mouse
							int lmb, // 0 ignore, 1 pressed, 2 released
							int rmb, // 0 ignore, 1 pressed, 2 released
							int mmb) // 0 ignore, 1 pressed, 2 released
{
	TheMouse.x = x;					 // Store mouse position
	TheMouse.y = y;

	if( lmb==1 )
	{
		TheMouse.lmb=1;
		TheMouse.xpress = x;
		TheMouse.ypress = y;
		ButtonPress(x,y);
	}
	if( lmb==2 )
	{
		TheMouse.lmb = 0;
		ButtonRelease(x,y);
	}

	if( rmb==1 ) TheMouse.rmb=1;
	if( rmb==2 ) TheMouse.rmb=0;


	ButtonPassive(x, y);					   // Do simple mouse testing
}//End MouseActions(..)





int UIButton::CreateButton(char *label,        // Button Name, e.g "Button1"
						   ButtonCallback cb,  // Callback Function
						   int x,int y,        // Top left position
						   int w,int h)        // Width & Height
{
	Button* p = (Button*)malloc( sizeof(Button) );
	assert(p);
	memset(p,0,sizeof(Button));
	p->x = x;
	p->y = y;
	p->w = w;
	p->h = h;
	p->callbackFunction = cb;
	p->label = (char*)malloc( strlen(label)+1 );
	if(p->label)
		sprintf(p->label,label);

	p->next = pButtonList;
	pButtonList = p;

	return p->id = ++GlobalRef;
}



int UIButton::DeleteButtonByName(char *label)
{
	Button* previous=NULL,*curr=pButtonList;
	while (curr!=NULL) {
		if (strcmp(label,curr->label)==0) {
			if(previous)
				previous->next = curr->next;
			else
				pButtonList = curr->next;

			if (curr->label) {

				free(curr->label);
			}
			free(curr);
			return 1;
		}
		previous = curr;
		curr = curr->next;
	}
	return 0;
}


int UIButton::DeleteButtonById(int id)
{
	Button	*previous = NULL       ,
			*curr     = pButtonList;

	while (curr!=NULL) 
	{
		if ( id == curr->id )
		{
			if(previous)
				previous->next = curr->next;
			else
				pButtonList = curr->next;

			if (curr->label) {

				free(curr->label);
			}
			free(curr);
			return 1;
		}
		previous = curr;
		curr = curr->next;
	}
	return 0;
}

int UIButton::ButtonClickTest(Button* b,int x,int y) 
{
	if( b) 
	{

		// If clicked within button area, then return true
	    if( x > b->x      && 
			x < b->x+b->w &&
			y > b->y      && 
			y < b->y+b->h ) {
				return 1;
		}
	}

	// Otherwise false.
	return 0;
}


void UIButton::ButtonRelease(int x,int y)
{
	Button* b = pButtonList;
	while(b) 
	{
		// If the mouse button was pressed within the button area
		// as well as being released on the button.....
		if( ButtonClickTest(b,TheMouse.xpress,TheMouse.ypress) && 
			ButtonClickTest(b,x,y) )
		{
			
			//	Then if a callback function has been set, call it.
			if (b->callbackFunction) {
				b->callbackFunction();
			}
		}


		// Set state back to zero.
		b->state = 0;

		b=b->next;
	}
}


void UIButton::ButtonPress(int x,int y)
{
	Button* b = pButtonList;
	while(b) 
	{
		// If the mouse click was within the buttons client area, 
		// set the state to true.
		if( ButtonClickTest(b,x,y) )
		{
			b->state = 1;
		}
		b=b->next;
	}
}


void UIButton::ButtonPassive(int x,int y)
{
	int needRedraw = 0;
	Button* b = pButtonList;
	while(b) 
	{
		// if the mouse moved over the control
		if( ButtonClickTest(b,x,y) )
		{
			//
			// If the cursor has just arrived over the control, set the highlighted flag
			// and force a redraw. The screen will not be redrawn again until the mouse
			// is no longer over this control
			//
			if( b->highlighted == 0 ) {
				b->highlighted = 1;
				needRedraw=1;
			}
		}
		else

		//
		// If the cursor is no longer over the control, then if the control
		// is highlighted (ie, the mouse has JUST moved off the control) then
		// we set the highlighting back to false, and force a redraw. 
		//
		if( b->highlighted == 1 )
		{
			b->highlighted = 0;
			needRedraw=1;
		}
		b=b->next;
	}
}



void UIButton::FlatSquare(IDirect3DDevice  * pDevice, 
				int x, 
				int y, 
				int w, 
				int h,
				unsigned int col )
{
		struct TLVERTEX
		{
			float x,y,z,rhw;
			unsigned int colour;
			enum{ FVF_TLVERTEX = D3DFVF_XYZRHW | D3DFVF_DIFFUSE };
		};

		pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
		pDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
		pDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);
		
		TLVERTEX Vertex[4] = 
		{
			//     x          y       z  rhw     colour
			{ (float)x,   (float)y,   0,  1.0f,   col},
			{ (float)x+w, (float)y,   0,  1.0f,   col},
			{ (float)x+w, (float)y+h, 0,  1.0f,   col},
			{ (float)x,   (float)y+h, 0,  1.0f,   col},
		};
		pDevice->SetTexture( 0, NULL);

		
#if(DIRECT3D_VERSION >= 0x0900)
		pDevice->SetFVF( TLVERTEX::FVF_TLVERTEX );
#else
		pDevice->SetVertexShader( TLVERTEX::FVF_TLVERTEX );
#endif // DIRECT3D_VERSION

		pDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, Vertex, sizeof( TLVERTEX ) );
}




void UIButton::WireEdge(IDirect3DDevice  * pDevice, 
					int x0, int y0,
					int x1, int y1,
					int x2, int y2,
					unsigned int col )
{
		struct TLVERTEX
		{
			float x,y,z,rhw;
			unsigned int colour;
			enum{ FVF_TLVERTEX = D3DFVF_XYZRHW | D3DFVF_DIFFUSE };
		};

		pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
		pDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
		pDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);
		
		TLVERTEX Vertex[3] = 
		{
			//   x            y         z  rhw     colour
			{ (float)x0,   (float)y0,   0,  1.0f,   col},
			{ (float)x1,   (float)y1,   0,  1.0f,   col},
			{ (float)x2,   (float)y2,   0,  1.0f,   col},
		};
		pDevice->SetTexture( 0, NULL);

		
#if(DIRECT3D_VERSION >= 0x0900)
		pDevice->SetFVF( TLVERTEX::FVF_TLVERTEX );
#else
		pDevice->SetVertexShader( TLVERTEX::FVF_TLVERTEX );
#endif // DIRECT3D_VERSION

		pDevice->DrawPrimitiveUP( D3DPT_LINESTRIP, 2, Vertex, sizeof( TLVERTEX ) );
}


void UIButton::TextOut(IDirect3DDevice * pDevice,
			char * str,			// null terminated string ("test")
			int xpos,			// top x pos
			int ypos,			// left y pos
			int w,				// width
			int h,				// height
			unsigned int rgb,	// aarrggbb
			int size)			// size (ie. 15)
{
	static ID3DXFont* pFont;

#if(DIRECT3D_VERSION >= 0x0900)

	D3DXCreateFont(pDevice,					// LPDIRECT3DDEVICE9
				size,						// Width
				0,							// Height
				FW_NORMAL,					// Weight
				1,							// MipLevels
				false,						// Italic
				DEFAULT_CHARSET,			// CharSet
				OUT_DEFAULT_PRECIS,			// OutputPrecision
				ANTIALIASED_QUALITY,		// Quality
				FIXED_PITCH | FF_DONTCARE,	// PitchAndFamily
				"Aerial",					// pFacename
				&pFont);					// LPD3DXFONT

#else
	static LOGFONT lfont={
			size,							// height
			0,								// width; 
			0,								// lfEscapement; 
			0,								// lfOrientation; 
			FW_BOLD,						// lfWeight; 
			FALSE,							// lfItalic; 
			FALSE,							// lfUnderline; 
			FALSE,							// lfStrikeOut; 
			DEFAULT_CHARSET,				// lfCharSet; 
			OUT_DEFAULT_PRECIS,				//lfOutPrecision; 
			CLIP_DEFAULT_PRECIS,			// lfClipPrecision; 
			ANTIALIASED_QUALITY,			// lfQuality; 
			FIXED_PITCH,					// lfPitchAndFamily; 
			"Aerial"						// lfFaceName[LF_FACESIZE]; 
			};

	lfont.lfHeight = size;

   //This is all there is to creating a D3DXFont.  Hmmmm....
   D3DXCreateFontIndirect( pDevice ,&lfont,&pFont );

#endif // DIRECT3D_VERSION



   // Temporary string buffer
   static char szbuff[900];
   sprintf(szbuff, "%s", str );
   RECT rect = {xpos,ypos, xpos+w,ypos+h};


#if(DIRECT3D_VERSION >= 0x0900)
   pFont->DrawTextA(NULL,					// pSprite
					szbuff,					// pString
					-1,						// Count
					&rect,					// pRect
					DT_CENTER|DT_VCENTER,	// Format	
					rgb);					// Color
#else
   pFont->DrawTextA(szbuff,					// pString
					-1,						// Count
					&rect,					// pRect
					DT_CENTER|DT_VCENTER,	// Format
					rgb);					// Color
#endif // DIRECT3D_VERSION

	pFont->Release();
}



//-------------------------------------------------------------------------//
//
// RenderSingleButton(..)
// For each button we call this function and it displays it to 
// the screen - taking into account if its in focus, alpha blend
// etc
//
//-------------------------------------------------------------------------//
void UIButton::RenderSingleButton(IDirect3DDevice * pDevice,
								  Button * b)
{
	pDevice->SetRenderState(  D3DRS_FILLMODE,D3DFILL_SOLID );

	pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	// SrcColor * SrcAlpha + DestColor * InvSrcAlphpa
	pDevice->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_SRCALPHA);
	pDevice->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_INVSRCALPHA);

	// AARRGGBB
	DWORD dwColour = D3DXCOLOR(0.6f,0.6f,0.6f,BUTTONALPHA);
	if (b->highlighted)
		dwColour = D3DXCOLOR(0.7f,0.7f,0.8f,BUTTONALPHA);

	FlatSquare(pDevice,
			    b->x,
				b->y,
				b->w,
				b->h,
				dwColour);

	dwColour = D3DXCOLOR(0.8f,0.8f,0.8f,BUTTONALPHA);
	if (b->state) 
		dwColour = D3DXCOLOR(0.4f,0.4f,0.4f,BUTTONALPHA);

	WireEdge(pDevice, 
				b->x+b->w, b->y,
				b->x     , b->y,
				b->x     , b->y+b->h,
				dwColour );

	
	dwColour = D3DXCOLOR(0.4f,0.4f,0.4f,BUTTONALPHA);
	if (b->state) 
		dwColour = D3DXCOLOR(0.8f,0.8f,0.8f,BUTTONALPHA);
		
	WireEdge(pDevice, 
				b->x,      b->y+b->h,
				b->x+b->w, b->y+b->h,
				b->x+b->w, b->y,
				dwColour );

	//
	//	Store the current top left position of our button, our TextOut(..)
	//  function will center it, but we can shift it slightly to give a
	//  shadow effect
	//
	int fontx = b->x;
	int fonty = b->y;

	//
	//	if the button is pressed, make it look as though the string has been 
	//  pushed down. It's just a visual thing to help with the overall look....
	//
	if (b->state) 
	{
		fontx+=2;
		fonty+=2;
	}

	//
	//	If the cursor is currently over the button we offset the text 
	//  string and draw a shadow
	//
	if(b->highlighted)
	{
		TextOut(pDevice,
				b->label,
				fontx,
				fonty,
				b->w,
				b->h,
				0xFF000000,
				BUTTON_FONT_SIZE);
		fontx--;
		fonty--;
	}

	
	TextOut(pDevice,
			b->label,
			fontx,
			fonty,
			b->w,
			b->h,
			0xFFFFFFFF,
			BUTTON_FONT_SIZE);

		
	pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, 
							 FALSE );
}//End RenderSingleButton(..)


void UIButton::ButtonDraw(IDirect3DDevice * pDevice)
{
	Button* b = pButtonList;
	while(b) 
	{
		RenderSingleButton(pDevice, b);

		b = b->next;
	}
}//End ButtonDraw(..)


