/***************************************************************************/
/*                                                                         */
/* File:   button.h                                                        */
/* Author: bkenwright@xbdev.net                                            */
/* Date:   19-12-2005 (xmas)                                               */
/*                                                                         */
/***************************************************************************/


#ifndef UIBUTTON_H
#define UIBUTTON_H


#define CRTDBG_MAP_ALLOC						// detecting memory leaks
#include <stdlib.h>
#include <crtdbg.h>

#include <assert.h>								// assert(..)


#pragma comment(lib, "D3d8.lib")				// directx library files
#pragma comment(lib, "D3dx8.lib")
#include <d3dx8.h>								// directx header files

#include <stdio.h>								// sprintf(..)


// Making the program compatable for directx8.1 and 9.0
#if(DIRECT3D_VERSION >= 0x0900)
#define			IDirect3DDevice					IDirect3DDevice9
#define			IDirect3DTexture				IDirect3DTexture9
#define			IDirect3DIndexBuffer			IDirect3DIndexBuffer9
#define			IDirect3DVertexBuffer			IDirect3DVertexBuffer9
#else
#define			IDirect3DDevice					IDirect3DDevice8
#define			IDirect3DTexture				IDirect3DTexture8
#define			IDirect3DIndexBuffer			IDirect3DIndexBuffer8
#define			IDirect3DVertexBuffer			IDirect3DVertexBuffer8
#endif // DIRECT3D_VERSION



//---------------------------------------------------------------------------


struct Mouse 
{
	int x;		// the x coordinate of the mouse cursor
	int y;		// the y coordinate of the mouse cursor
	int lmb;	// is the left button pressed?
	int mmb;	// is the middle button pressed?
	int rmb;	// is the right button pressed?

	// When you press the mouse button and drag it around the screen, these 
	// variables keep a record of the initial button press position.

	int xpress; // stores the x-coord of when the first button press occurred
	int ypress;	// stores the y-coord of when the first button press occurred
};





typedef void (*ButtonCallback)();


struct Button 
{
	int   x;							// top left x coord of the button
	int   y;							// top left y coord of the button
	int   w;							// the width of the button
	int   h;							// the height of the button
	int	  state;						// the state, 1 if pressed, 0 otherwise
	int	  highlighted;					// is the mouse cursor over the control?
	char* label;						// the text label of the button
	ButtonCallback callbackFunction;	// A pointer to a function to call if the button is pressed

	int id;								// A unique ID to represent this Button

	struct Button* next;				// a pointer to the next node in the linked list
};


// Forward declaration of our class (full definition at the bottom)
class UIAutoButtonDestroy;


class UIButton
{
	
		static Mouse TheMouse;


		static Button* pButtonList;

		static int GlobalRef;

		static UIAutoButtonDestroy CleanUp;

public:
		static void DestoyAllButtons()
		{
			for(int i=GlobalRef; i>=0; i--)
				DeleteButtonById(i);
		}


		static void MouseActions(int x, int y, int lmb, int rmb, int mmb);



		static int CreateButton(char *label,ButtonCallback cb,int x,int y,int w,int h);


		static int DeleteButtonByName(char *label);

		static int DeleteButtonById(int id);

public:
		static void GetMousePosition(int * x, int * y)
		{
			*x = TheMouse.x;
			*y = TheMouse.y;
		}

		static int IsLeftButtonPressed()
		{
			return TheMouse.lmb;
		}
		static int IsRightButtonPressed()
		{
			return TheMouse.rmb;
		}

		static int AnyButtonsActive()
		{
			int nActive = 0;
			Button* b = pButtonList;
			while(b) 
			{
				if( b->highlighted )
					nActive++;
				b=b->next;
			}
			return nActive;
		}
protected:

		static int ButtonClickTest(Button* b,int x,int y);


		static void ButtonRelease(int x,int y);


		static void ButtonPress(int x,int y);


		static void ButtonPassive(int x,int y);


		// These three function are that use device dependent code! - So
		// in effect if we rewrite these three function for opengl, then our
		// UIButton class should work easily on opengl
		static void FlatSquare(IDirect3DDevice  * pDevice, 
						int x, 
						int y, 
						int w, 
						int h,
						unsigned int col );


		static void WireEdge(IDirect3DDevice  * pDevice, 
							int x0, int y0,
							int x1, int y1,
							int x2, int y2,
							unsigned int col );


		static void TextOut(IDirect3DDevice * pDevice,
					char * str,							// button caption
					int xpos,							// left pos
					int ypos,							// top pos 
					int w,								// width button
					int h,								// height button
					unsigned int rgb,					// colour aarrggbb
					int size);							// font size

		static void RenderSingleButton(IDirect3DDevice * pDevice,
								       Button * p);

public:
		// Called from our render loop and draws all our buttons
		static void ButtonDraw(IDirect3DDevice * pDevice);
};



// Because our class UButton uses lots of static variables - C++ doesn't have
// a specific static destructor.  So any memory allocated in our static functions
// won't be release! 
// So by creating a single instance of this class as static, in our UIButton class,
// it means that it will be called once when the program ends, and it will release
// any resources that where allocated in our UIButton class
class UIAutoButtonDestroy
{
public:
	~UIAutoButtonDestroy()
	{
		UIButton::DestoyAllButtons();
	}
};



#endif // UIBUTTON_H

