/***************************************************************************/
/*                                                                         */
/* File:   dxdraw.cpp                                                      */
/* Author: bkenwright@xbdev.net                                            */
/* Date:   19-12-2005                                                      */
/*                                                                         */
/***************************************************************************/


#pragma comment(lib, "kernel32.lib")
#pragma comment(lib, "user32.lib")
#pragma comment(lib, "gdi32.lib")
#pragma comment(lib, "comctl32.lib")
#pragma comment(lib, "Advapi32.lib")
#pragma comment(lib, "Winmm.lib")

// Use visual studio to detect memory leaks in our application
#define CRTDBG_MAP_ALLOC			// detecting memory leaks
#include <stdlib.h>
#include <crtdbg.h>

#include <windows.h>				// various windows definitions
#include <stdio.h>					// printf(..)

#define ULONG_PTR unsigned long

#pragma comment(lib, "D3d8.lib")	// directx library files
#pragma comment(lib, "D3dx8.lib")
#include <d3dx8.h>					// directx header files


#include "dxdbg.h"

#include "button.h"					// UIButton class
	
#include "model.h"					// float model[] data

#include "common.h"					// SafeRelease(..)

#define					BUILD_VERSION		"0.0.01b"

LPDIRECT3D8				g_pD3D				= NULL;
LPDIRECT3DDEVICE8		g_pD3DDevice		= NULL;

HWND					g_hwnd				= 0;


static UINT				g_RenderMode		= D3DFILL_WIREFRAME;
static bool				g_bStats			= false;
static bool				g_bCross			= false;
static bool				g_bWireGround		= false;

static float			g_fSpinX			= 0.0f;
static float			g_fSpinY			= 90.0f;
static float			g_fZoom				= 190.0f;

static int				g_nMouseX			= 0.0f;
static int				g_nMouseY			= 0.0f;



//-------------------------------------------------------------------------//
//                    Button CallBack Functions                            //
//-------------------------------------------------------------------------//

static void RenderModeFunc()
{
	//D3DFILL_POINT               = 1
	//D3DFILL_WIREFRAME           = 2
	//D3DFILL_SOLID               = 3

	g_RenderMode += 1;
	if( g_RenderMode >= 4 ) g_RenderMode=2;
}

static void StatsFunc()
{
	g_bStats = !g_bStats;
}

static void CrossFunc()
{
	g_bCross  =!g_bCross;
}

static void WireGround()
{
	g_bWireGround = !g_bWireGround;
}

//-------------------------------------------------------------------------//
//        Creation/Destruction of our DirectX Applications                 //
//-------------------------------------------------------------------------//

bool Create(HWND hWnd)
{
	g_hwnd = hWnd;
    //First of all, create the main D3D object. If it is created successfully 
	//we should get a pointer to an IDirect3D8 interface.
    g_pD3D = Direct3DCreate8(D3D_SDK_VERSION);
	if( g_pD3D==NULL )
	{
		ErrMsg("Error Creating DirectX Object Instance");
		return false;
	}

    //Get the current display mode
    D3DDISPLAYMODE d3ddm;
	HRESULT hr = 
    g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);
	if( hr!=S_OK )
	{
		ErrMsg("Failed Getting Display Mode");
		return false;
	}

    //Create a structure to hold the settings for our device
    D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory(&d3dpp, sizeof(d3dpp));
    //Fill the structure. 
    //We want our program to be windowed, and set the back buffer to a format
    //that matches our current display mode
    d3dpp.Windowed				 = TRUE;
    d3dpp.SwapEffect			 = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat		 = d3ddm.Format;
    //For depth buffering (e.g.) the z-buffer
    d3dpp.BackBufferCount		 = 1;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
    d3dpp.EnableAutoDepthStencil = TRUE;
    
	//Create a Direct3D device
	hr =														// Error Checking
    g_pD3D->CreateDevice(D3DADAPTER_DEFAULT,					// UINT Adapter
						 D3DDEVTYPE_HAL,						// D3DDEVTYPE DeviceType
						 hWnd,									// HWND hFocusWindow
						 D3DCREATE_SOFTWARE_VERTEXPROCESSING,	// DWORD BehaviorFlags
						 &d3dpp,								// D3DPRESENT_PARAMETERS *
						 &g_pD3DDevice);						// IDirect3DDevice9 *
	if( hr!=S_OK )
	{
		ErrMsg("Failed Creating Rendering Device");
		return false;
	}


    //Turn on z-buffering
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);

	//Set our culling mode to Counter Clock Wise
	g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);


	//Craete our buttons and hand them our button handler functions
	UIButton::CreateButton("Cross",		  CrossFunc,       5,50, 120,25);
	UIButton::CreateButton("RenderMode",  RenderModeFunc,  5,90, 120,25);
	UIButton::CreateButton("Stats",		  StatsFunc,	   5,130,120,25);
	UIButton::CreateButton("Wire Horizon",WireGround,	   5,170,120,25);
	

	return true;
}//End Create(..)


bool Release()
{
	SafeRelease( g_pD3DDevice );

	SafeRelease( g_pD3D );

	return true;
}//End Release()

//-------------------------------------------------------------------------//

void SetMaterial()
{
      D3DMATERIAL8 matMaterial;
      D3DCOLORVALUE rgbaDiffuse  = {1.0, 1.0, 1.0, 0.0};  //RGBA
      D3DCOLORVALUE rgbaAmbient  = {1.0, 1.0, 1.0, 0.0};
      D3DCOLORVALUE rgbaSpecular = {0.0, 0.0, 0.0, 0.0};
      // emissive is for a glow effect
      D3DCOLORVALUE rgbaEmissive = {0.0, 0.0, 0.0, 0.0};

      matMaterial.Diffuse  = rgbaDiffuse;
      matMaterial.Ambient  = rgbaAmbient;
      matMaterial.Specular = rgbaSpecular;
      matMaterial.Emissive = rgbaEmissive;
      matMaterial.Power    = 100.0f;

      g_pD3DDevice->SetMaterial(&matMaterial);
}//End SetMaterial(..)

 

void SetLight()
{
	//Set a light up?
    D3DLIGHT8 d3dLight;

    //Initialize the light structure.
    ZeroMemory(&d3dLight, sizeof(D3DLIGHT8));

    //Set up a white point light at (0, 0, -10).
    d3dLight.Type			= D3DLIGHT_POINT;

    // Actual colour of the light. (e.g. white).
    d3dLight.Diffuse.r		= 1.0f;
    d3dLight.Diffuse.g		= 1.0f;
    d3dLight.Diffuse.b		= 1.0f;

    // Colour when no light hits the object.
    d3dLight.Ambient.r		= 0.2f;
    d3dLight.Ambient.g		= 0.2f;
    d3dLight.Ambient.b		= 0.2f;

    // Shiny effect.
    d3dLight.Specular.r		= 0.1f;
    d3dLight.Specular.g     = 0.1f;
    d3dLight.Specular.b     = 0.1f;

    d3dLight.Position.x     = 0.0f;
    d3dLight.Position.y     = 0.0f;
    d3dLight.Position.z     = -30.0f;

    d3dLight.Attenuation0	= 1.0f; 
    d3dLight.Attenuation1	= 0.0f; 
    d3dLight.Attenuation2	= 0.0f; 
    d3dLight.Range			= 100.0f;

    g_pD3DDevice->SetLight(0, &d3dLight);
    g_pD3DDevice->LightEnable(0,TRUE);

    //Turn on lighting
    g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
    g_pD3DDevice->SetRenderState(D3DRS_AMBIENT, 
								 D3DCOLOR_XRGB(20,20,20));
}//End SetLight(..)



//---------------------------------------------------------------------------
//
// MouseInput(..)
// For this demo, our mouse data is collected by the static class UIButton
// in button.h.  So that our buttons still work correctly, and when we are
// in focus of our buttons, our app window doesn't get mouse input, we need
// to ask our Button class if its okay to use the mouse information and 
// if a button was pressed.
//
//---------------------------------------------------------------------------
void MouseInput()
{
	int xpos = 0;
	int ypos = 0;

	// We can use static variables here, so we don't have to do any globals
	static int xposold = xpos;
	static int yposold = ypos;

	UIButton::GetMousePosition(&xpos, &ypos);

	g_nMouseX = xpos;
	g_nMouseY = ypos;

	int dx = xpos - xposold;
	int dy = ypos - yposold;

	xposold = xpos;
	yposold = ypos;

	// No buttons have the focus
	if( UIButton::AnyButtonsActive() == 0 )
	{

		if( UIButton::IsLeftButtonPressed() )
		{
			g_fSpinX += 0.5f*dx;
			g_fSpinY -= 0.5f*dy;

			g_fSpinX = static_cast<int>(g_fSpinX) % 360;
			g_fSpinY = static_cast<int>(g_fSpinY) % 360;

		}
		if( UIButton::IsRightButtonPressed() )
		{
			g_fZoom -= dy;

		}
	}
}//End MouseMove(..)



void SetCamera()
{
	D3DXVECTOR3 vCamPosition = D3DXVECTOR3( 0, 90, -g_fZoom );

    // [1] D3DTS_VIEW
    D3DXMATRIX v;
    g_pD3DDevice->SetTransform(	D3DTS_VIEW, 
								D3DXMatrixLookAtLH(&v, 
								&vCamPosition, 
                                &D3DXVECTOR3(0,0,0), 
                                &D3DXVECTOR3(0,1,0)));  
    // [2] D3DTS_PROJECTION
    D3DXMATRIX p;
    g_pD3DDevice->SetTransform( D3DTS_PROJECTION, 
								D3DXMatrixPerspectiveFovLH( &p,  
								D3DX_PI/4,  
								1.0f,
                                1.0f,500.0f));
}//End SetCamera(..)


void DrawCross()
{
	// Get the window size
	RECT Rect;
	GetClientRect(g_hwnd, &Rect);

	//
	// Large cross across the screen which changes as the mouse moves
	//
	// Vertical Line
	DrawLine(g_pD3DDevice,									// DirectX Device
			 0,												// x-start
			 g_nMouseY,										// y-start
			 Rect.right,									// x-end
			 g_nMouseY,										// y-end
			 0x99999999);									// Colour

	// Horizontal Line
	DrawLine(g_pD3DDevice,									// DirectX Device
			 g_nMouseX,										// x-start
			 0,												// y-start
			 g_nMouseX,										// x-end
			 Rect.bottom,									// y-end
			 0x9999999);									// Colour

	//
	// Lets draw a littl cross in the middle of the screen
	//
	// Vertical Line
	int xCenter = Rect.right/2;
	int yCenter = Rect.bottom/2;
	int nSize = 50;
	DrawLine(g_pD3DDevice,									// DirectX Device
			 xCenter-nSize,									// x-start
			 yCenter,										// y-start
			 xCenter+nSize,									// x-end
			 yCenter,										// y-end
			 0x444444FF);									// Colour

	// Horizontal Line
	DrawLine(g_pD3DDevice,									// DirectX Device
			 xCenter,										// x-start
			 yCenter-nSize,									// y-start
			 xCenter,										// x-end
			 yCenter+nSize,									// y-end
			 0x444444FF);									// Colour

}//End DrawCross(..)



//---------------------------------------------------------------------------
//
// DrawSolidModel(..)
// Not a very efficient function - as the original model was jsut rendered 
// in wire frame - so it didn't need any modification of our raw data from 
// our model file.  But when rendered in solid mode it looked just like a 
// black block, and couldn't see the details.  So had to parse the the data 
// to add colour information and normals for lighting.
//
//---------------------------------------------------------------------------
// ** Performance Floor ** Performance Floor ** Performance Floor ** Perfo //
//---------------------------------------------------------------------------
void DrawSolidModel()
{
	D3DXMATRIX matRot;
	D3DXMatrixRotationYawPitchRoll( &matRot,				// pOut
								D3DXToRadian(g_fSpinX),		// Yaw
								D3DXToRadian(g_fSpinY),		// Pitch
								0.0f );						// Roll

    g_pD3DDevice->SetTransform(	   D3DTS_WORLD, 
								   &matRot );

	g_pD3DDevice->SetRenderState(  D3DRS_FILLMODE, 
								   D3DFILL_SOLID );

	g_pD3DDevice->SetVertexShader( D3DFVF_XYZ |				// Position
								   D3DFVF_NORMAL |			// Normal (for lighting)
								   D3DFVF_DIFFUSE);			// Colour

	struct stV
	{
		D3DXVECTOR3 p; //Position
		D3DXVECTOR3 n; //Normal
		DWORD       c; //Colour
	};

	// 5300 triangles
	// 5300 * 3 = 15900 vertices
	// Allocated a temporary block of memory for our new 3d model data
	stV * pVerts = new stV[15900];

	// Lets copy across all the triangles
	int c = 0;
	for(int i=0; i<5300 * 3; i++)
	{
		pVerts[i].p.x = model[c++];
		pVerts[i].p.y = model[c++];
		pVerts[i].p.z = model[c++];
		pVerts[i].c  = 0xFF444444;
	}

	//CalculateNormals
	c=0;
	for(int i=0; i<5300; i++)
	{
		D3DXVECTOR3 pA = pVerts[c++].p;
		D3DXVECTOR3 pB = pVerts[c++].p;
		D3DXVECTOR3 pC = pVerts[c++].p;

		D3DXVECTOR3 N;

		D3DXVec3Cross(&N, &(pB-pA), &(pC-pA) );
		D3DXVec3Normalize(&N, &N);

		pVerts[c-3].n = pVerts[c-2].n = pVerts[c-1].n = N;
	}



	//Render our data to the directx device
	g_pD3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLELIST, 
								   5300, 
								   pVerts, 
								   sizeof( stV ) );
	//Release temporaty data
	delete[] pVerts;

}//End DrawSolidModel(..)



void DrawWireModel()
{

	D3DXMATRIX matRot;
	D3DXMatrixRotationYawPitchRoll( &matRot,				// pOut
								D3DXToRadian(g_fSpinX),		// Yaw
								D3DXToRadian(g_fSpinY),		// Pitch
								0.0f );						// Roll

    g_pD3DDevice->SetTransform(	   D3DTS_WORLD,				// Set World Matrix
								   &matRot );				// Position Model

	g_pD3DDevice->SetRenderState(  D3DRS_FILLMODE,			// Type of fill mode
								   D3DFILL_WIREFRAME);

	g_pD3DDevice->SetVertexShader( D3DFVF_XYZ );

	g_pD3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLELIST,		// PrimitiveType
								   5300,					// PrimitiveCount
								   model,					// pVertexStreamZeroData
								   sizeof( D3DXVECTOR3 ) ); // VertexStreamZeroStride
}//End DrawWireModel(..)


void DrawModel()
{
	if( g_RenderMode == D3DFILL_WIREFRAME )
		DrawWireModel();
	else
		DrawSolidModel();
}//End DrawModel()


void Render()
{
	if(!g_pD3DDevice)return;

	SetCamera();

	SetMaterial();

	SetLight();

	MouseInput();

	// Clear the back buffer to a blue color
	g_pD3DDevice->Clear( 0, 
						 NULL, 
						 D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 
						 D3DCOLOR_XRGB(0,0,255), 
						 1.0f, 
						 0 );


	g_pD3DDevice->BeginScene();


	DrawModel();

	// Using a static variable so we don't need a global
	static debugtxt txt(g_pD3DDevice);

	// Display information to the user on the screen
	if( g_bStats )
	{
		// Get the window size
		RECT rect;
		GetClientRect(g_hwnd, &rect);

		int yl = rect.bottom - 20;
		int yr = yl;
		int xr = rect.right  - 150;
		txt.printf("Rendering 5300 Triangles", 10, yl ); yl-=20;

		char buf[200]; // large temp txt buffer
		sprintf(buf, "X-Rot:%.2f  Y-Rot:%.2f", g_fSpinX, g_fSpinY);
		txt.printf(buf, 10, yl ); yl-=20;

		sprintf(buf, "Z-Zoom:%.2f", g_fZoom);
		txt.printf(buf, 10, yl ); yl-=20;

		switch(g_RenderMode)
		{
		case D3DFILL_WIREFRAME:
			sprintf(buf, "Render Mode: %s", "WIRE" ); break;
		case D3DFILL_SOLID:
			sprintf(buf, "Render Mode: %s", "SOLID" );break;
		case D3DFILL_POINT:
			sprintf(buf, "Render Mode: %s", "POINT" );break;
		}
		txt.printf(buf, 10, yl ); yl-=20;

		sprintf(buf, "Mouse x:%d  y:%d", g_nMouseX, g_nMouseY);
		txt.printf(buf, 10, yl ); yl-=20;


		sprintf(buf, "Compiled: %s", __DATE__);
		txt.printf(buf, xr, yr ); yr-=20;

		sprintf(buf, "Build: %s", BUILD_VERSION);
		txt.printf(buf, xr, yr ); yr-=20;

	}

	if( g_bCross ) DrawCross();


	UIButton::ButtonDraw(g_pD3DDevice);

	
	static debug_wire_ground ground(g_pD3DDevice, 30, 30);
	if( g_bWireGround )
		ground.Draw(g_pD3DDevice, 0,0,0, 200 );


	FPSDisplay(g_pD3DDevice);
	g_pD3DDevice->EndScene();
	// After rendering the scene we display it.
	g_pD3DDevice->Present( NULL, NULL, NULL, NULL );
	
}




