/***************************************************************************/
/*                                                                         */
/* File:   main.cpp                                                        */
/* Author: bkenwright@xbdev.net                                            */
/* Date:   19-12-2005 (xmas)                                               */
/*                                                                         */
/***************************************************************************/


// These few lines are compiler directives that include the windows librarys
// during linking.  You could instead inside visual studio goto Project->
// settings" menu and under the "Link" tab add the necessary librarys instead.
#pragma comment(lib, "kernel32.lib")
#pragma comment(lib, "user32.lib")
#pragma comment(lib, "gdi32.lib")
#pragma comment(lib, "comctl32.lib")
#pragma comment(lib, "Advapi32.lib")
#pragma comment(lib, "Winmm.lib")

// Visual Studio Debugging - Always best to check for memory leaks!  You
// never know, we all make mistakes.  So you declare these variables before
// any of the others and of course call:
// _CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
// At the start of our program - then any memory we didnt release and
// visual studio will tell us when our program exits.
#define CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <windows.h>					//Various Windows Function
#include <stdio.h>						//sprintf(...)

#include "button.h"						//UIButton class

#include "dxdbg.h"						//ErrorMessage(..)

#include "resource.h"					//Icon Resource

// So our DirectX Code doesn't get mixed with our Win32 Code we put the
// all the DirectX Creation,Rendering and Destruction code in another
// file.
extern bool Create(HWND hWnd);			// draw.cpp
extern bool Release();					// draw.cpp
extern void Render();					// draw.cpp



//---------------------------------------------------------------------------
void GameLoop()
{
	Render();
}

//---------------------------------------------------------------------------
LRESULT CALLBACK WndProcedure(HWND hWnd, UINT uMsg,
			      WPARAM wParam, LPARAM lParam);
//---------------------------------------------------------------------------
INT WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
	_CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );

	MSG        Msg;
	HWND       hWnd;
	WNDCLASSEX WndClsEx;
    const char *ClsName = "www.xbdev.net - Demo - by bkenwright@xbdev.net";
   	const char *WndName = ClsName;

	// Create the application window
	WndClsEx.cbSize        = sizeof(WNDCLASSEX);
	WndClsEx.style         = CS_HREDRAW | CS_VREDRAW;
	WndClsEx.lpfnWndProc   = WndProcedure;
	WndClsEx.cbClsExtra    = 0;
	WndClsEx.cbWndExtra    = 0;
	WndClsEx.hIcon         = LoadIcon(hInstance, 
							 MAKEINTRESOURCE(IDI_ICON));
	WndClsEx.hCursor       = LoadCursor(NULL, IDC_ARROW);
	WndClsEx.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	WndClsEx.lpszMenuName  = NULL;
	WndClsEx.lpszClassName = ClsName;
	WndClsEx.hInstance     = hInstance;
	WndClsEx.hIconSm       = LoadIcon(hInstance, 
							 MAKEINTRESOURCE(IDI_ICON));


	// Register the application
	RegisterClassEx(&WndClsEx);

	// Create the window object
	hWnd = CreateWindowEx(0,
                          ClsName,
                          WndName,
               			  WS_SYSMENU | WS_MINIMIZEBOX,
                          CW_USEDEFAULT,
               			  CW_USEDEFAULT,
                          CW_USEDEFAULT,
               			  CW_USEDEFAULT,
                          NULL,
                          NULL,
                          hInstance,
                          NULL);
	
	// Find out if the window was created
	if( !hWnd ) // If the window was not created,
	{
		ErrMsg("Problem Creating Window (CreateWindowEx(..)");
		return FALSE; // stop the application
	}

	if( Create(hWnd) == false )
	{
		ErrMsg("Failed to Create and Start DirectX Application");
		Release();
		return 0;
	}

	// Display the window to the user
	ShowWindow(hWnd, nCmdShow);// SW_SHOWNORMAL);
	UpdateWindow(hWnd);

	// Decode and treat the messages
	// as long as the application is running
	while(TRUE)
	{
		
		if (PeekMessage(&Msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if(!GetMessage(&Msg, 0,0,0))
				break;
			DispatchMessage(&Msg);
		}
		else
			GameLoop();	
	}

	Release();

//	return Msg.wParam;
    return 0;
}
//---------------------------------------------------------------------------
LRESULT CALLBACK WndProcedure(HWND hWnd, UINT Msg,
			   WPARAM wParam, LPARAM lParam)
{
    switch(Msg)
    {
	case WM_KEYDOWN:
        if( wParam==VK_ESCAPE )PostQuitMessage(0);
		break;

	case WM_LBUTTONDOWN:
		UIButton::MouseActions(LOWORD(lParam), HIWORD (lParam), 1, 0, 0);
		break;
	case WM_RBUTTONDOWN:
		UIButton::MouseActions(LOWORD(lParam), HIWORD (lParam), 0, 1, 0);
		break;
	case WM_LBUTTONUP:
		UIButton::MouseActions(LOWORD(lParam), HIWORD (lParam), 2, 0, 0);
		break;
	case WM_RBUTTONUP:
		UIButton::MouseActions(LOWORD(lParam), HIWORD (lParam), 0, 2, 0);
		break;
	case WM_MOUSEMOVE:
		UIButton::MouseActions(LOWORD(lParam), HIWORD (lParam), 0, 0, 0);
		break;


    case WM_DESTROY:
        PostQuitMessage(WM_QUIT);
        break;
    default:
        // Process the left-over messages
        return DefWindowProc(hWnd, Msg, wParam, lParam);
    }
    // If something was not done, let it go
    return 0;
}
//---------------------------------------------------------------------------



